/*
    ftremote - remote serial/bitbang for FT232 and similar chips
    Copyright (C) 2002-2009  Ladislav Vaiz <ok1zia@nagano.cz>
    and authors of web browser Links 0.96

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

#include "header.h"

//#include <sched.h>

int retval = RET_OK;
long int starting_sbrk;
int opt_d, opt_g, opt_h, opt_i, opt_s, opt_t, opt_r, opt_w, debug_keyboard=0;

char *opt_hostname=NULL;

void usage(){
    printf(
"\n"            
" ftremote - Remote FTDI chip control\n"
" Copyright (C) 2009 Ladislav Vaiz <ok1zia@nagano.cz\n"
"\n"
" Command line arguments:\n"
" -d debug mode\n"
" -h this help plus configuration file format\n"
" -i informations about HW etc\n"
" -s server mode\n"
" -v version\n"
"\n");
}

char *parse_options(int argc, char **argv){
    char *s,c;

    s=getenv("FTREMOTE_DEBUG");
    if (s){
        if (strlen(s)!=0){
            debug_type=1;
            debug_filename=s;
        }else{
            debug_type=2;
        }
    }
#ifdef HAVE_GETOPT_H    
    
    while (1){
        
        int option_index = 0;
        static struct option long_options[] = {
            {"debug", 2, 0, 'd'},
            {"help", 0, 0, 'h'},
            {"version", 0, 0, 'v'},
            {0, 0, 0, 0}
        };

        c = getopt_long (argc, argv, "dghikrstvw?", long_options, &option_index);
        if (c == -1) break;

        switch (c){
            case 0:
/*              printf ("option %s", long_options[option_index].name);*/
                if (optarg) printf (" with arg %s", optarg);
                printf ("\n");
                break;

            case 'd':
                opt_d++;
                if (optarg) {
                    debug_type=1;
                    debug_filename=optarg;
                }else{
                    debug_type=2;
                }
                break;

            case 'g':
                opt_g++;
                break;
                
            case 'h':     
            case '?':
                opt_h++;
                break;

            case 'i':
                opt_i++;
                break;

            case 'k':
                debug_keyboard=1;
                break;

            case 'r':
                opt_r++;
                break;
                            
            case 's':
                opt_s++;
                break;
           
            case 't':
                opt_t++;
                break;
           
            case 'v':
                /*printf ("option v\n");*/
                break;

            case 'w':
                opt_w++;
                break;


            default:
                printf ("?? getopt returned character code 0%o ??\n", c);
        }
    }
    if (optind < argc){
        opt_hostname = argv[optind++];
    }

    if (optind < argc){
        printf ("non-option ARGV-elements: ");
        while (optind < argc) printf ("%s ", argv[optind++]);
        printf ("\n");
    }

#endif    
    return NULL;
}


    
void sig_terminate(union cba_t cba)
{
    unhandle_basic_signals();
    terminate = 1;
    retval = RET_SIGNAL;
}

void sig_intr(union cba_t cba)
{              
    unhandle_basic_signals();
    terminate = 1;
}

void sig_ign(union cba_t cba)
{
}



void sig_segv(union cba_t cba){
    signal(SIGSEGV, SIG_DFL);
    fprintf(stderr, "sig_segv(%d)", cba.int_);
/*    printf("\n\n\n  sig_segv() sdl=%p \n\n\n",sdl);*/
/*  install_signal_handler(SIGSEGV, NULL, CBA0, 0);*/
    raise(SIGSEGV);
}   

void sig_pipe(int a){
   /* dbg("sig_pipe(%d)\n", a);
    dbg("pid=%d\n", getpid());
    sleep(10);*/
}
    
void handle_basic_signals()
{
#ifndef HAVE_SDL    
#define sdl 0
#endif
    install_signal_handler(SIGHUP, sig_intr, CBA0, 0);
    install_signal_handler(SIGTERM, sig_terminate, CBA0, 0);
#ifdef SIGTTOU
    install_signal_handler(SIGTTOU, sig_ign, CBA0, 0);
#endif
    install_signal_handler(SIGSEGV, sig_segv, CBA0, 1);
	signal(SIGPIPE, sig_pipe);
}



void unhandle_terminal_signals()
{
    install_signal_handler(SIGHUP, NULL, CBA0, 0);
#ifdef SIGTSTP
    install_signal_handler(SIGTSTP, NULL, CBA0, 0);
#endif
#ifdef SIGTTIN
    install_signal_handler(SIGTTIN, NULL, CBA0, 0);
#endif
#ifdef SIGCONT
    install_signal_handler(SIGCONT, NULL, CBA0, 0);
#endif
#ifdef SIGTTOU
    install_signal_handler(SIGTTOU, NULL, CBA0, 0);
#endif
    install_signal_handler(SIGSEGV, NULL, CBA0, 0);
	signal(SIGPIPE, SIG_IGN);
}

void unhandle_basic_signals()
{
    install_signal_handler(SIGHUP, NULL, CBA0, 0);
    install_signal_handler(SIGTERM, NULL, CBA0, 0);
#ifdef SIGTSTP
    install_signal_handler(SIGTSTP, NULL, CBA0, 0);
#endif
#ifdef SIGTTIN
    install_signal_handler(SIGTTIN, NULL, CBA0, 0);
#endif
#ifdef SIGCONT
    install_signal_handler(SIGCONT, NULL, CBA0, 0);
#endif
#ifdef SIGTTOU
    install_signal_handler(SIGTTOU, NULL, CBA0, 0);
#endif
    install_signal_handler(SIGSEGV, NULL, CBA0, 0);
	signal(SIGPIPE, SIG_IGN);
}

#ifndef HAVE_SDL
#undef sdl
#endif


/*struct status dump_stat;*/
int dump_pos;

int ac;
char **av;

char *path_to_exe;

/*int init_b = 0;*/

char *ftremote_home = NULL;
int first_use = 0;
int first_contest_def = 0;

static inline int dir_sep(char x) { return x == '/'; }

#if 0
char *get_home(int *n)
{
    struct stat st;
    char *home = stracpy(getenv("HOME"));
    char *home_ftremote;
    char *config_dir = stracpy(getenv("CONFIG_DIR"));

    if (n) *n = 1;
    if (!home) {
        int i;
        home = stracpy(path_to_exe);
        if (!home) {
            if (config_dir) mem_free(config_dir);
            return NULL;
        }
        for (i = strlen(home) - 1; i >= 0; i--) if (dir_sep(home[i])) {
            home[i + 1] = 0;
            goto br;
        }
        home[0] = 0;
        br:;
    }
    while (home[0] && dir_sep(home[strlen(home) - 1])) home[strlen(home) - 1] = 0;
    if (home[0]) add_to_strn(&home, "/");
    home_ftremote = stracpy(home);
    if (config_dir)     
    {
        add_to_strn(&home_ftremote, config_dir);
        while (home_ftremote[0] && dir_sep(home_ftremote[strlen(home_ftremote) - 1])) home_ftremote[strlen(home_ftremote) - 1] = 0;
        if (stat(home_ftremote, &st) != -1 && S_ISDIR(st.st_mode)) {
            add_to_strn(&home_ftremote, "/ftremote");
            } else {
            fprintf(stderr, "CONFIG_DIR set to %s. But directory %s doesn't exist.\n\007", config_dir, home_ftremote);
            sleep(3);
            mem_free(home_ftremote);
            home_ftremote = stracpy(home);
            add_to_strn(&home_ftremote, "ftremote");        
        }
        mem_free(config_dir);
    } else add_to_strn(&home_ftremote, "ftremote");
    if (stat(home_ftremote, &st)) {
        if (!mkdir(home_ftremote, 0777)) goto home_creat;
        if (config_dir) goto failed;
        goto first_failed;
    }
    if (S_ISDIR(st.st_mode)) goto home_ok;
    first_failed:
    mem_free(home_ftremote);
    home_ftremote = stracpy(home);
    add_to_strn(&home_ftremote, "ftremote");
    if (stat(home_ftremote, &st)) {
        if (!mkdir(home_ftremote, 0777)) goto home_creat;
        goto failed;
    }
    if (S_ISDIR(st.st_mode)) goto home_ok;
    failed:
    mem_free(home_ftremote);
    mem_free(home);
    return NULL;

    home_ok:
    if (n) *n = 0;
    home_creat:
#ifdef HAVE_CHMOD
    chmod(home_ftremote, 0700);
#endif
    add_to_strn(&home_ftremote, "/");
    mem_free(home);
    return home_ftremote;
}

void init_home(void)
{
    ftremote_home = get_home(&first_use);
    if (!ftremote_home) {
        fprintf(stderr, "Unable to find or create ftremote config directory. Please check, that you have $HOME variable set correctly and that you have write permission to your home directory.\n\007");
        sleep(3);
        return;
    }
}
#endif

void init(void)
{
    int tcpport;

    parse_options(ac, av);
    if (opt_h){
        usage();
		printf("\nConfiguration file /etc/ftremote or ~/.ftremote has format:\n"
				"key value\n\n"
				"Keys are:\n"
				"%s", help);
        exit(0);
    }
    if (opt_i){
        printf("\n  %s %s\n\n", PACKAGE, VERSION);
        printf("%s\n", txt_settings);
/*#ifdef __GLIBC__
        printf("\n  glibc info\n");
        printf("compiled version: %d.%d\n", __GLIBC__, __GLIBC_MINOR__);
        printf("runtime version: %s\n", gnu_get_libc_version());
        printf("runtime release: %s\n\n", gnu_get_libc_release());
#endif */
        
        read_rc_files();
        rc_info();
#ifdef HAVE_LIBFTDI
        usb_info();
#endif
        exit(0);
    }
        
//    g_thread_init(NULL);
    init_debug();
    init_rc();
	
    //init_home();
	
	read_rc_files();
    
    
    if (ftdi_open(1)) terminate = 1;

    if (opt_s){
        // server
        init_tcp();
    }else{
        // client
        if (!opt_hostname){
            opt_hostname = rc_getopt("server"); /* Hostname/IP of server to connect to */
            if (!opt_hostname){
                usage();
                exit(0);
            }
        }
        tcpport=rc_int("tcpport", 7300); /* TCP port to connect */
        tcp_connect(opt_hostname, tcpport);
    }
    	
    handle_basic_signals(); 

}

void terminate_all_subsystems(void)
{
    dbg("terminate_all_subsystems\n");
    check_bottom_halves();
    check_bottom_halves();
	check_bottom_halves();

    free_ftdi();

    shrink_memory(1);

    check_bottom_halves();
    check_bottom_halves();

    if (ftremote_home) mem_free(ftremote_home);

    free_rc();
    check_memory_leaks();
    free_debug();
}


int main(int argc, char *argv[])
{
//    g_thread_init(NULL);

    path_to_exe = argv[0];
    ac = argc;
    av = (char **)argv;

    starting_sbrk = (long int)sbrk;
/*    dbg("\n\n");*/
    select_loop(init);
    terminate_all_subsystems();

    
    return retval;
}


void shrink_memory(int u)
{
/*  shrink_dns_cache(u);
    shrink_format_cache(u);
    garbage_collection(u);
    delete_unused_format_cache_entries();*/
}
