/*
    ftremote - remote serial/bitbang for FT232 and similar chips
    Copyright (C) 2002-2009  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

 
#include "header.h"

#define RC_USER "/.ftremote"



struct rcitem cfg = { &cfg, &cfg };

int init_rc(){
}

int free_rc(){
}

int read_rc_file(char *filename){
    FILE *f;
    char s[1024], *c, *ss;
    int len;
    struct rcitem *rci, *rcj;

    dbg("read_rc_file('%s')\n", filename);
    f = fopen(filename, "rt");
    if (!f) return -1;

    while (fgets(s, 1000, f) != NULL){
        len = strlen(s);
        if (len > 0 && s[len - 1] == '\n'){
            s[len - 1] = '\0';
            len--;
        }
        if (len > 0 && s[len - 1] == '\r'){ 
            s[len - 1] = '\0'; /* probably unreachable, fgets() should strip \r */
            len--;
        }

		for (ss=s; *ss==' ' || *ss=='\t'; ss++); 
	  //  dbg("s='%s', ss='%s'\n", s, ss);
		
        c = index(ss, '#');
		if (c) *c='\0';

        c = index(ss, ' ');
        if (!c) continue;
		if (c==ss) continue;

        rci = NULL;
        foreach(rcj, cfg){
            if (strlen(rcj->key) != c - ss) continue;
            if (strncmp(ss, rcj->key, c - ss) == 0) {
                rci = rcj;
                break;
            }
        }

        if (!rci){
            rci = mem_alloc(sizeof(struct rcitem));
            rci->key = strndup(ss, c - ss);
            add_to_list(cfg, rci);
        }else{
            mem_free(rci->value);
        }
        rci->value = strdup(c + 1);
	    dbg("key='%s', value='%s'\n", rci->key, rci->value);
    }
    fclose(f);    
    
    return 0;
}


int read_rc_files(void){
    char *user, *home;

    if (read_rc_file("/etc/ftremote")==0) return;

    home = getenv("HOME");
    if (!home) return 0;
    user = mem_alloc(strlen(home)+strlen(RC_USER)+1);
    strcpy(user, home);
    strcat(user, RC_USER);
    read_rc_file(user);
    mem_free(user);
    return 0;
}

char *rc_getopt(char *key){
    struct rcitem *rci;

    foreach(rci, cfg){
        if (strcmp(key, rci->key) != 0) continue;
        return rci->value;
    }
    return NULL;
}

char *rc_str(char *key, char *def){
    char *ret;
    ret = rc_getopt(key);
    if (!ret) ret = def;
    return ret;
}

int rc_int(char *key, int def){
    char *c;
    int ret;
    
    c = rc_getopt(key);
    if (!c) ret = def;
    else ret = strtol(c, NULL, 0);

    return ret;
}

int rc_double(char *key, double def){
    char *c;
    double ret;
    
    c = rc_getopt(key);
    if (!c) ret = def;
    else ret = atoi(c);

    return ret;
}

void rc_info(){
    struct rcitem *rci;

    printf("\n  rc_info:\n");
    foreach(rci, cfg){
        printf("key='%s'\tvalue='%s'\n", rci->key, rci->value);
    }
}


#if 0
gint save_rc_file(gchar *filename){
    FILE *f;
    GString *gs;
    int ret;
    
    gs = g_string_sized_new(10000);
    
    f=fopen(filename,"wt");
    if (!f) {
/*        c = g_strconcat("Can't open file '", filename, "'", NULL);
        errbox(c,0);
        g_free(c);*/
        return errno;
    }

    save_rc_string(gs);
    ret = fprintf(f, "%s", gs->str) != gs->len ? errno:0;
    fclose(f);
    g_string_free(gs, TRUE);

    return ret;
}
#endif


